﻿package MT::Plugin::MeCab;

use strict;
use MT;
use MT::Plugin;

our $VERSION = '0.2';
our $SCHEMA_VERSION = 0.01;

use base qw( MT::Plugin );

@MT::Plugin::MeCab::ISA = qw( MT::Plugin );

my $plugin = __PACKAGE__->new( {
    id  => 'MeCab',
    key => 'MeCab',
    description => 'Auto Tagging use MeCab',
    name => 'MeCab',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    version => $VERSION,
    schema_version => $SCHEMA_VERSION,
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        callbacks => {
            'cms_post_save.entry'
                => \&_cms_post_save_entry,
            'cms_post_save.page'
                => \&_cms_post_save_entry,
        },
        object_types => {
            'objecttag' => {
                'score'     => 'integer',
            },
        },
        tags => {
            modifier => {
                'kanji2kana' => \&_kanji2kana,
            },
        },
    } );
}

sub _kanji2kana {
    my( $str, $arg, $ctx ) = @_;
    eval { require MeCab };
    unless ( $@ ){
        my $mecab = MeCab::Tagger->new();
        my $node = $mecab->parseToNode( $str );
        my @words;
        for ( ; $node; $node = $node->{ next } ) {
            next unless defined $node->{ surface };
            my $parse = $node->{ feature };
            push ( @words, $parse );
        }
        my %count;
        @words = grep { !$count{ $_ }++ } @words;
        for my $parse ( @words ) {
            my @item = split ( /,/, $parse );
            my $search = quotemeta ( $item[6] );
            my $replace = $item[7];
            $str =~ s/$search/$replace/g;
        }
        return $str;
    } else {
        MT->log( 'MeCab is not available' );
        return $str;
    }
}

sub _cms_post_save_entry {
    my ( $cb, $app, $obj, $original ) = @_;
    eval { require MeCab };
    unless ( $@ ){
        use Encode qw( decode );
        my $str = $obj->title . ' ' . $obj->title . ' ' . $obj->text;
        $str =~ s/<script.{1,}?<\/script>//sg;
        $str =~ s/<!\-\-[^>]*?\-\->//sg;
        $str =~ s/<img[^>]*?alt=*["']([^"'>]*)["'][^>]*?>/$1/gi;
        $str =~ s/<[^>]*?>//sg;
        $str =~ s/[\n|\t]//sg;
        $str =~ s/"//sg;
        my $mecab = MeCab::Tagger->new();
        my $node = $mecab->parseToNode( $str );
        my @tags; # = $obj->tags;
        unless ( defined $original ) {
            @tags = $obj->tags;
        }
        my @words;
        for ( ; $node; $node = $node->{ next } ) {
            next unless defined $node->{ surface };
            # my $surface = $node->{ surface };
            my $parse = $node->{ feature };
            # $app->log( $parse );
            if ( ( $parse =~ /^名詞,一般,/ ) ||
                 ( $parse =~ /^名詞,固有名詞,組織,/ ) ) {
                my @item = split ( /,/, $parse );
                my $word = $item[6];
                next if ( $word =~ /会社$/ );
                my $length;
                if ( MT->version_number >= 5 ) {
                    $length = length( $word );
                } else {
                    $length = length( decode( 'utf-8', $word ) );
                }
                if ( $parse =~ /Tags$/ ) {
                    push ( @tags, $word );
                } else {
                    if ( ( $length > 7 ) && ( $word ne '*' ) ) {
                        push ( @words, $word );
                    }
                }
            }
        }
        my %count_tags;
        my %count_words;
        my %tag;
        for my $key ( @tags ) {
            $count_tags{ $key } = $count_tags{ $key } + 1;
        }
        for my $key ( keys %count_tags ) {
            $tag{ $key } = $count_tags{ $key };
        }
        for my $key ( @words ) {
            $count_words{ $key } = $count_words{ $key } + 1;
        }
        for my $key ( sort{ $count_words{ $b } cmp $count_words{ $a } } keys %count_words ) {
            if ( $count_words{ $key } > 1 ) {
                $tag{ '@' . $key } = $count_tags{ $key };
            } else {
                last;
            }
        }
        _save_tags( $obj, \%tag );
        # $obj->save or die $obj->errstr;
    } else {
        $app->log( 'MeCab is not available' );
    }
}

sub _save_tags {
    my ( $obj, $tags ) = @_;
    require MT::ObjectTag;
    my $clear_cache = 0;
    my $t = MT->get_timer;
    $t->pause_partial if $t;
    $obj->{__tag_objects} = [];
    my $blog_id = $obj->has_column( 'blog_id' ) ? $obj->blog_id : 0;
    my @existing_tags = MT::ObjectTag->load( { object_id => $obj->id,
        object_datasource => $obj->datasource } );
    my %existing_tags = map { $_->tag_id => $_ } @existing_tags;
    for my $tag_name ( keys %$tags ) {
        my $tag_count = $tags->{ $tag_name };
        my $tag = MT::Tag->load( { name => $tag_name },
            { binary => { name => 1 } } );
        if ( $tag ) {
            if ( exists $existing_tags{ $tag->id } ) {
                $existing_tags{ $tag->id } = 0;
                push @{ $obj->{ __tag_objects } }, $tag;
                next;
            }
        } else {
            # new tag
            $tag = new MT::Tag;
            $tag->name( $tag_name );
            $tag->save or next;
            $clear_cache = 1;
        }
        my $otag = new MT::ObjectTag;
        $otag->blog_id( $blog_id );
        $otag->tag_id( $tag->id );
        $otag->object_id( $obj->id );
        $otag->object_datasource( $obj->datasource );
        $otag->score( $tag_count );
        $otag->save or return $obj->error( $otag->errstr );
        $existing_tags{ $tag->id } = 0;
        $clear_cache = 1;
        push @{ $obj->{ __tag_objects } }, $tag;
    }
    for my $otag ( values %existing_tags ) {
        next unless ref $otag;
        my $this_tag_id = $otag->tag_id;
        $otag->remove;
        if (! MT::ObjectTag->exist({ tag_id => $this_tag_id })) {
            if ( my $tag = MT::Tag->load( $this_tag_id ) ) {
                $tag->remove;
            }
        }
        $clear_cache = 1;
    }
    delete $obj->{ __save_tags };
    if ( $clear_cache ) {
        MT::Tag->clear_cache( datasource => $obj->datasource, ( $blog_id ? ( blog_id => $blog_id) : () ) );
        require MT::Memcached;
        MT::Memcached->instance->delete( $obj->tag_cache_key );
    }
    $t->mark( 'MT::Tag::save_tags' ) if $t;
    1;
}

1;